#ifndef _COM_										//    
#define _COM_

#include <avr/iom128.h>								//  /
#include <avr/interrupt.h>							//  

#ifndef F_CPU
# warning "F_CPU not defined. Frequency by default 14.7456 MHz"
#define F_CPU 14745600UL							/*     (     )*/
#endif
#define _TIMER_DIV 8								/*    1  3*/

#include <util/delay.h>								//   

#define ENABLE	1									/* "   DCON "*/
#define DISABLE	0									/* "   DCON "*/
#define DCON	0									/* " DCON"*/
#define MODBUS	1									/* " Modbus RTU"*/
#define COM_PORT1	0								/* " COM1"*/
#define COM_PORT2	1								/* " COM2"*/

#ifndef COM_RX_BUFFER_SIZE							
# warning "COM_RX_BUFFER_SIZE not defined. Value will be set by default 64 bytes"
#define COM_RX_BUFFER_SIZE 64						/*  */
#endif
#ifndef COM_TX_BUFFER_SIZE							
# warning "COM_TX_BUFFER_SIZE not defined. Value will be set by default 64 bytes"
#define COM_TX_BUFFER_SIZE 64						/*  */
#endif

char _CHK_mode[2]={0,0};							// /     DCON
char _Protocol[2]={0,0};							// (0-DCON/1-Modbus RTU)
char _Frame_pause[2];								//   1.5   Modbus RTU (1- /0-  )


static char _UART_RX_Buf[2][COM_RX_BUFFER_SIZE];	//  UART
static char _UART_RX_dup[2][COM_RX_BUFFER_SIZE];	//   UART
static int _UART_RX_point[2];						//     UART
static char _UART_RX_length[2];						//    UART
static char _UART_RX_end[2];						//     UART

static char _UART_TX_Buf[2][COM_TX_BUFFER_SIZE];	//  UART
static int _UART_TX_point[2];						//     UART
static char _UART_TX_length[2];						//    UART
static char _UART_TX_end[2];						//    UART

#ifdef COM1_MODBUS
//------------------------------------------------------------------------------------------
//========================      1 ============================
//------------------------------------------------------------------------------------------
SIGNAL(TIMER1_COMPA_vect)
{
	int temp;																// 
	
	_Frame_pause[0]=0;														//    1.5 
	
	for (temp=0; temp<_UART_RX_point[0]; temp++) _UART_RX_dup[0][temp]=_UART_RX_Buf[0][temp];//  
	_UART_RX_length[0]=_UART_RX_point[0];								    //   
	_UART_RX_point[0]=0;										            //    
	TCCR1B=0x00;															// 
	_UART_RX_end[0]=1;														//   
}

//------------------------------------------------------------------------------------------
//========================    B  1 ============================
//------------------------------------------------------------------------------------------
SIGNAL(TIMER1_COMPB_vect)
{
	_Frame_pause[0]=1;														//    1.5 
}
#endif

//===============================    UART0 ===============================
SIGNAL(USART0_RX_vect)
{
	if (_Protocol[0]==MODBUS)
	{
#ifdef COM1_MODBUS
		
//----------------------------------   Modbus ----------------------------------
		if (_Frame_pause[0]==0)                                             //     1.5                        
		{
			_UART_RX_Buf[0][_UART_RX_point[0]]=UDR0;						//  
			TIFR|=(1<<OCF1A)|(1<<OCF1B);                                    //     (        )
			TCNT1=0;                                                        //   
			TCCR1B=(1<<WGM12)|(1<<CS11);									//    "  "     8
			_UART_RX_point[0]++;                                            //   
			if (_UART_RX_point[0]>=COM_RX_BUFFER_SIZE-1) _UART_RX_point[0]=0;//    
		}
#endif
	}
	else
//-----------------------------------   DCON -----------------------------------
	{
		char temp;															// 
		
		_UART_RX_Buf[0][_UART_RX_point[0]]=UDR0;							//  

		switch (_UART_RX_Buf[0][_UART_RX_point[0]])                         //   ,     
		{
			case '$':
			case '%':
			case '~':
			case '@':
			case '#':
			case '^':
			case '!':
			case '?':
			case '>':
			temp=_UART_RX_Buf[0][_UART_RX_point[0]];						//     
			_UART_RX_point[0]=0;											//       
			_UART_RX_Buf[0][0]=temp;										//      
		}
	
		if (_UART_RX_Buf[0][_UART_RX_point[0]]==0x0D)						//    
		{
			
			_UART_RX_length[0]=_UART_RX_point[0]+1;							//  

			//  
			for (_UART_RX_point[0]=0; _UART_RX_point[0]<=_UART_RX_length[0]; _UART_RX_point[0]++) _UART_RX_dup[0][_UART_RX_point[0]]=_UART_RX_Buf[0][_UART_RX_point[0]];
			_UART_RX_point[0]=0;											//   
			_UART_RX_end[0]=1;												//   
		}
		else
		{
			_UART_RX_point[0]++;											//   
			if (_UART_RX_point[0]>=COM_RX_BUFFER_SIZE-1) _UART_RX_point[0]=0;//    
		}
	}
}

//=============================    UART0 ===============================
SIGNAL(USART0_TX_vect)
{
	if (_UART_TX_point[0]==_UART_TX_length[0])								//   
	{
		_UART_TX_point[0]=0;												//   
		PORTG&=~(1<<PG0);													// RS485  
		_UART_TX_end[0]=1;													//   
	}
	else
	{
		UDR0=_UART_TX_Buf[0][_UART_TX_point[0]];							//   UART0
		_UART_TX_point[0]++;												//   
	}
}

#ifdef COM2_MODBUS
//------------------------------------------------------------------------------------------
//========================      3 ============================
//------------------------------------------------------------------------------------------
SIGNAL(TIMER3_COMPA_vect)
{
	int temp;																// 
	
	_Frame_pause[1]=0;														//    1.5 
	
	for (temp=0; temp<_UART_RX_point[1]; temp++) _UART_RX_dup[1][temp]=_UART_RX_Buf[1][temp];//  
	_UART_RX_length[1]=_UART_RX_point[1];								    //   
	_UART_RX_point[1]=0;										            //    
	TCCR3B=0x00;															// 
	_UART_RX_end[1]=1;														//   
}

//------------------------------------------------------------------------------------------
//========================    B  3 ============================
//------------------------------------------------------------------------------------------
SIGNAL(TIMER3_COMPB_vect)
{
	_Frame_pause[1]=1;														//    1.5 
}
#endif

//===============================    UART1 ===============================
SIGNAL(USART1_RX_vect)
{
	if (_Protocol[1]==MODBUS)
	{
#ifdef COM2_MODBUS
//----------------------------------   Modbus ----------------------------------
		if (_Frame_pause[1]==0)                                             //     1.5 
		{
			_UART_RX_Buf[1][_UART_RX_point[1]]=UDR1;						//  
			ETIFR|=(1<<OCF3A)|(1<<OCF3B);                                   //     (        )
			TCNT3=0;                                                        //   
			TCCR3B=(1<<WGM32)|(1<<CS31);									//    "  "     8
			_UART_RX_point[1]++;                                            //   
			if (_UART_RX_point[1]>=COM_RX_BUFFER_SIZE-1) _UART_RX_point[1]=0;//    
		}
#endif
	}
	else
//-----------------------------------   DCON -----------------------------------
	{
		char temp;															// 

		_UART_RX_Buf[1][_UART_RX_point[1]]=UDR1;							//  

		switch (_UART_RX_Buf[1][_UART_RX_point[1]])                         //   ,     
		{
			case '$':
			case '%':
			case '~':
			case '@':
			case '#':
			case '^':
			case '!':
			case '?':
			case '>':
			temp=_UART_RX_Buf[1][_UART_RX_point[1]];						//     
			_UART_RX_point[1]=0;											//       
			_UART_RX_Buf[1][0]=temp;										//      
		}
		
		if (_UART_RX_Buf[1][_UART_RX_point[1]]==0x0D) 						//    
		{
			_UART_RX_length[1]=_UART_RX_point[1]+1;							//  

			for (_UART_RX_point[1]=0; _UART_RX_point[1]<=_UART_RX_length[1]; _UART_RX_point[1]++) _UART_RX_dup[1][_UART_RX_point[1]]=_UART_RX_Buf[1][_UART_RX_point[1]];//  
			_UART_RX_point[1]=0;											//   
			_UART_RX_end[1]=1;												//   
		}
		else
		{
			_UART_RX_point[1]++;											//   
			if (_UART_RX_point[1]>=COM_RX_BUFFER_SIZE-1) _UART_RX_point[1]=0;//    
		}
	}
}

//=============================    UART1 ===============================
SIGNAL(USART1_TX_vect)
{
	if (_UART_TX_point[1]==_UART_TX_length[1])								//   
	{
		_UART_TX_point[1]=0;												//   
		PORTG&=~(1<<PG1);													// RS485  
		_UART_TX_end[1]=1;													//   
	}
	else
	{
		UDR1=_UART_TX_Buf[1][_UART_TX_point[1]];							//   UART0
		_UART_TX_point[1]++;												//   
	}
}

//===========================   COM  ==============================
//    :
//port -  COM 
//COM_speed -    UART
//Protocol -   (0-DCON/1-Modbus RTU)
// :
//0x00-    ,    0xFF

char COM_Init(int port, long COM_speed, char Protocol)
{
	if (port>1) return 0xFF;												//     COM 
	if ((COM_speed<=0) || (COM_speed>1000000)) return 0xFF;					//    
	if (Protocol>1) return 0xFF;											//     /  
	
	_Protocol[port]=Protocol;												// 
		
	float temp;
	
	temp=(float)F_CPU/COM_speed/16-0.5;										//    UART
	if (temp>=0.5)															//     
	{
		if (port==0)
		{
			UBRR0H=(unsigned long)temp>>8;
			UBRR0L=(unsigned long)temp & 0x000000FF;
		}
		else 
		{
			UBRR1H=(unsigned long)temp>>8;
			UBRR1L=(unsigned long)temp & 0x000000FF;
		}		
	}
	else return 0xFF;														//,   

	if (COM_speed>=19200) COM_speed=19200;									//   19200 /  ,    1.75 .
	if (port==0)
	{	
		UCSR0B=(1<<RXCIE0)|(1<<RXEN0)|(1<<TXEN0)|(1<<TXCIE0);				// -
		if (Protocol==MODBUS) UCSR0C=(1<<UCSZ00)|(1<<UCSZ01)|(1<<USBS0);	//  (8N2)   Modbus RTU
		else UCSR0C=(1<<UCSZ00)|(1<<UCSZ01);								//  (8N1)
		DDRG|=(1<<PG0);														//    RS485
		PORTG&=~(1<<PG0);													// UART0  

		_UART_RX_point[port]=0;												//   
		_UART_RX_end[port]=0;												//   
		_UART_TX_end[port]=1;												//   
		
#ifdef COM1_MODBUS
		TCCR1A=0;
		OCR1A=35*(float)F_CPU/COM_speed/_TIMER_DIV+0.5;						//  3.5 
		OCR1B=15*(float)F_CPU/COM_speed/_TIMER_DIV+0.5;						//  1.5 
		TIMSK|=(1<<OCIE1A)|(1<<OCIE1B);										//      B  1		
		sei();																//  		
#endif
	}
	else
	{
		UCSR1B=(1<<RXCIE1)|(1<<RXEN1)|(1<<TXEN1)|(1<<TXCIE1);				// -
		if (Protocol==MODBUS) UCSR1C=(1<<UCSZ10)|(1<<UCSZ11)|(1<<USBS1);	//  (8N2)   Modbus RTU
		else UCSR1C=(1<<UCSZ10)|(1<<UCSZ11);									//  (8N1)
		DDRG|=(1<<PG1);														//    RS485
		PORTG&=~(1<<PG1);													// UART1  

		_UART_RX_point[port]=0;												//   
		_UART_RX_end[port]=0;												//   
		_UART_TX_end[port]=1;												//   
		
#ifdef COM2_MODBUS
		TCCR3A=0;
		OCR3A=35*(float)F_CPU/COM_speed/_TIMER_DIV+0.5;						//  3.5 
		OCR3B=15*(float)F_CPU/COM_speed/_TIMER_DIV+0.5;						//  1.5 
		ETIMSK|=(1<<OCIE3A)|(1<<OCIE3B);									//      B  3
		sei();																//  
#endif
	}
	return 0;
}

//============  /    DCON ===============
//    :
//port -  COM 
//CHK -  /    DCON
// :
//0x00-   DCON       ,    0xFF

char CHK_Control(int port, char CHK)
{
	if (port>1) return 0xFF;												//     COM 
	if ((CHK>1) || (_Protocol[port]==1)) return 0xFF;						//     /       DCON
	_CHK_mode[port]=CHK;													//     DCON
	return 0;
}

//====================      UART =========================
//    :
//port -  COM 
//Len -   

void _UART_Go(char port, int Len)
{
	if (port==0)															//  COM1
	{
		_UART_TX_end[0]=0;													//    (  )
		PORTG|=(1<<PG0);													// RS485  
		_delay_us(5);														//    
		_UART_TX_length[0]=Len;												//  
		_UART_TX_point[0]=1;												//   
		UDR0=_UART_TX_Buf[0][0];											//  UART0   (    )
	}
	else																	//  COM2
	{
		_UART_TX_end[1]=0;													//    (  )

		PORTG|=(1<<PG1);													// RS485  
		_delay_us(5);														//    

		_UART_TX_length[1]=Len;												//  
		_UART_TX_point[1]=1;												//   
		UDR1=_UART_TX_Buf[1][0];											//  UART1   (    )		
	}
}

//=======================      DCON ==========================
// :
//port -  COM 
//*buf -     
// :
//0x00-  COM    ,    0xFF

char Write_DCON(int port, char *buf)
{
	if (port>1) return 0xFF;												//     COM 
	char CHK_sum=0;															//    
	char temp;																// 
	int Lengtch;															// 
	
	while (_UART_TX_end[port]==0) _delay_us(5);								//   
	
	Lengtch=0;																//     
	while (Lengtch<COM_TX_BUFFER_SIZE)										//,     ,       
	{
		_UART_TX_Buf[port][Lengtch]=*buf;
		if ((_UART_TX_Buf[port][Lengtch]==0) || (_UART_TX_Buf[port][Lengtch]==0x0D)) break;//        
		CHK_sum+=_UART_TX_Buf[port][Lengtch];
		Lengtch++;															//    
		buf++;																//     
	}
	
	if (_CHK_mode[port]==ENABLE)											//       DCON
	{
		temp=CHK_sum>>4;													//   
		if (temp<=9) _UART_TX_Buf[port][Lengtch]=temp+0x30;					//   (0-9)
		else _UART_TX_Buf[port][Lengtch]=temp+0x37;							//   (A-F)
		Lengtch++;
		
		temp=CHK_sum & 0x0F;												//   
		if (temp<=9) _UART_TX_Buf[port][Lengtch]=temp+0x30;					//   (0-9)
		else _UART_TX_Buf[port][Lengtch]=temp+0x37;							//   (A-F)
		Lengtch++;
	}

	_UART_TX_Buf[port][Lengtch++]=0x0D;										//      
	
	_UART_Go(port,Lengtch);													//    UART
	return 0;
}

//=======================      DCON ==========================
//      UART
// :
//port -  COM 
//*buf-        
// :
//	0x00-   UART ;
//	0x01-         ;
//	0x80-   (  Modbus);
//	0xFF-     .
//     ,       .
//       .

char Read_DCON(int port, char *buf)
{
	int temp;																// 
	char CHK_sum=0;															//    
	
	if (_UART_RX_end[port]==0) return 0;									//      
	if (_Protocol[port]!=DCON) return 0x80;									//   Modbus
	_UART_RX_end[port]=0;													//   
	
	if (_CHK_mode[port]==ENABLE)											//       DCON
	{
		for (temp=0;temp<_UART_RX_length[port]-3;temp++) CHK_sum+=_UART_RX_dup[port][temp];//    ,   CR    
		temp=CHK_sum>>4;													//   
		if (temp<=9) temp+=0x30;											//   (0-9)
		else temp+=0x37;													//   (A-F)
		if (temp!=_UART_RX_dup[port][_UART_RX_length[port]-3]) return 0xFF;	//       
		
		temp=CHK_sum & 0x0F;												//   
		if (temp<=9) temp+=0x30;											//   (0-9)
		else temp+=0x37;													//   (A-F)
		if (temp!=_UART_RX_dup[port][_UART_RX_length[port]-2]) return 0xFF;	//       
	}
	
	*buf=_UART_RX_length[port];												//   
	for (temp=0; temp<_UART_RX_length[port]; temp++)						// 
	{
		buf++;
		*buf=_UART_RX_dup[port][temp];
	}
	
	return 1;
}

//=================       Modbus RTU ===============
// :
//*buf-          
//Len-      
int _CRC_calc(char *buf, char Len)
{
	unsigned int CRC;														//    
	char temp,temp2;														// 
	
	CRC=0xFFFF;																//  CRC (   CRC)
	for (temp=0; temp<Len; temp++)											//      
	{
		CRC=CRC^*buf;														//" " (   CRC)
		buf++;																//   
		for (temp2=0; temp2<8; temp2++)										//     
		if ((CRC & 0x0001)==0) CRC=CRC>>1;									//    ,   (   CRC)
		else
		{
			CRC=CRC>>1;														//	 " " (   CRC)
			CRC=CRC^0xA001;													//  " " 1010 0000 0000 0001
		}
	}
	
	return CRC;																//   
}

//================       Modbus RTU ==================
// :
//port -  COM 
//*buf-   
// :
//	0x00-   UART ;
//	0x01-    UART     ;
//	0x80-   (  DCON);
//	0xFF-     .
//     ,       .
//       .
//       ,    .

char Read_Modbus(int port, char *buf)
{
	unsigned int CRC;														//   
	int temp;
	
	if (_UART_RX_end[port]==0) return 0;									//      
	
	if (_Protocol[port]!=MODBUS) return 0x80;								//   DCON	
	_UART_RX_end[port]=0;													//   
	
	CRC=_CRC_calc(_UART_RX_Buf[port],_UART_RX_length[port]-2);				// CRC

	//  
	if (((CRC>>8)!=_UART_RX_Buf[port][_UART_RX_length[port]-1]) || ((CRC & 0xFF)!=_UART_RX_Buf[port][_UART_RX_length[port]-2])) return 0xFF;

	*buf=_UART_RX_length[port];												//     
	for (temp=0; temp<_UART_RX_length[port]; temp++)						// 
	{
		buf++;
		*buf=_UART_RX_Buf[port][temp];
	}
	
	
	return 1;
}

//===============        Modbus RTU ================
// :
//Addres- 
//Func- 
//SubFunc-  ( )
//Data- 
//Amount-   (  )
// :
//	0x00-  
//	0xFE-   
//	0xFF-   COM 

char Write_Response_Modbus(int port, char Addres, char Func, unsigned int SubFunc, char *Data, unsigned int Amount)
{
	unsigned int CRC;																			//   
	int Lengtch;																				// 
	char temp;																					// 
	char _Error=0;																				//    
	
	if (port>1) return 0xFF;																	//   COM    
	while (_UART_TX_end[port]==0) _delay_us(5);														//   
	
	Lengtch=0;
	_UART_TX_Buf[port][Lengtch++]=Addres;														// 
	_UART_TX_Buf[port][Lengtch++]=Func;															// 
	
	if ((Func & 0x80)==0)																		//     
	{
		switch(Func)
		{
			case 0x01:
			case 0x02:
			Amount=((Amount-1)>>3)+1;															//   
			if (Amount>(COM_TX_BUFFER_SIZE-5)/2) {_Error=1; break;}
			_UART_TX_Buf[port][Lengtch++]=Amount;
			for (temp=0; temp<Amount; temp++)
			{
				_UART_TX_Buf[port][Lengtch++]=*Data;											//  
				Data++;																			//   
			}
			break;

			case 0x03:
			case 0x04:
			if (Amount>(COM_TX_BUFFER_SIZE-5)/2) {_Error=1; break;}
			Amount<<=1;																			//   
			_UART_TX_Buf[port][Lengtch++]=Amount;
			for (temp=0; temp<Amount; temp++)
			{
				_UART_TX_Buf[port][Lengtch++]=*Data;											//  
				Data++;																			//   
			}
			break;
			
			case 0x05:
			case 0x06:
			_UART_TX_Buf[port][Lengtch++]=SubFunc>>8;											//   
			_UART_TX_Buf[port][Lengtch++]=SubFunc & 0xFF;										//   
			
			_UART_TX_Buf[port][Lengtch++]=*Data;												//   
			Data++;
			_UART_TX_Buf[port][Lengtch++]=*Data;												//   
			break;
			
			case 0x0F:
			case 0x10:
			_UART_TX_Buf[port][Lengtch++]=SubFunc>>8;											//   
			_UART_TX_Buf[port][Lengtch++]=SubFunc & 0xFF;										//   
			
			_UART_TX_Buf[port][Lengtch++]=Amount>>8;											//   
			_UART_TX_Buf[port][Lengtch++]=Amount &0xFF;											//   
			break;
			
			default: return 0xFE;																// ,     
		}
	}
	else _UART_TX_Buf[port][Lengtch++]=*Data;													//    ,   
	
	if (_Error==0)
	{
		CRC=_CRC_calc(_UART_TX_Buf[port],Lengtch);												//  
		
		_UART_TX_Buf[port][Lengtch++]=CRC & 0xFF;												//   
		_UART_TX_Buf[port][Lengtch++]=CRC>>8;													//   
		
		_UART_Go(port,Lengtch);																	//    UART
		return 0;																				//  
	}
	else return 1;
}

//====================      Modbus RTU =====================
// :
//port- COM 
//Addres- 
//Func- 
//SubFunc-  ( )
//*Data- 
//Amount-   (  )

void Write_Request_Modbus(int port, char Addres, char Func, unsigned int SubFunc, char *Data, unsigned int Amount)
{
	unsigned int CRC;														//   
	int Lengtch;															// 
	char temp;																// 
	
	while (_UART_TX_end[port]==0) _delay_us(5);								//   
	if (((Func<0x01) || (Func>0x06)) && (Func!=0x0F) && (Func!=0x10)) return; //,     
	
	Lengtch=0;
	_UART_TX_Buf[port][Lengtch++]=Addres;									// 
	_UART_TX_Buf[port][Lengtch++]=Func;										// 
	_UART_TX_Buf[port][Lengtch++]=SubFunc>>8;								//   
	_UART_TX_Buf[port][Lengtch++]=SubFunc & 0xFF;							//   

	if ((Func==0x0F) || (Func==0x10))										//  0x0F  0x10.
	{
		_UART_TX_Buf[port][Lengtch++]=Amount>>8;							//   
		_UART_TX_Buf[port][Lengtch++]=Amount & 0xFF;						//   
		
		if (Func==0x10) Amount<<=1;											//   
		else 
			if (Func==0x0F) Amount=((Amount-1)>>3)+1;
			else return;
	
		_UART_TX_Buf[port][Lengtch++]=Amount;								//     
		
		for (temp=0; temp<Amount; temp++) 
		{
			_UART_TX_Buf[port][Lengtch++]=*Data;							//  
			Data++;															//   
		}
	}
	else
	{
		_UART_TX_Buf[port][Lengtch++]=*Data;								//   
		Data++;
		_UART_TX_Buf[port][Lengtch++]=*Data;								//   
	}
	
	CRC=_CRC_calc(_UART_TX_Buf[port],Lengtch);								//  
	
	_UART_TX_Buf[port][Lengtch++]=CRC & 0xFF;								//   
	_UART_TX_Buf[port][Lengtch++]=CRC>>8;									//   
	
	_UART_Go(port, Lengtch);												//    UART
}
#endif