/*
 * ADS1248.c
 *
 *  Created on: May 6, 2021
 *      Author: dubovik
 */


#include "ADS1248.h"

#include "avr/interrupt.h"

int32_t adc_buf;

uint32_t OFC1, OFC0, FSC1, FSC0;
coef_t ADCCOFF;

signed int Pt138_Grad[22]= {	
	-200,	-170,	-136,	-100,	-59,
	-12,	0,		38,		85,		100,
	131,	180,	200,	228,	275,
	321,	367,	414,	466,	510,
	554,	600
	};
	
float Pt138_Ohm[22]=       {
	1.8520000e+001,	3.1340000e+001,	4.5530000e+001,	6.0260000e+001,	7.6730000e+001,
	9.5300000e+001,	1.0000000e+002,	1.1477000e+002,	1.3280000e+002,	1.3851000e+002,
	1.5021000e+002,	1.6848000e+002,	1.7586000e+002,	1.8611000e+002,	2.0311000e+002,
	2.1951000e+002,	2.3566000e+002,	2.5191000e+002,	2.6959000e+002,	2.8430000e+002,
	2.9880000e+002, 3.1371000e+002
	};

#define SPI_ON		SPI_PORTX &= ~(1<<SPI_SS_ADS);\
					_delay_us(50);
#define SPI_OFF		_delay_us(50);\
					SPI_PORTX |= (1<<SPI_SS_ADS);


//функция поиска в таблице значения температуры по значению сопротивления
//float calk - значение температуры
//float *Ohm - ссылка на таблицу сопротивления
//int *Grad   - ссылка на таблицу температуры
//char count - количество элементов из таблицы учавствующие в поиске
//float x	- значение сопротивления
//int j		- номер элемента из таблицы с которого начанается поиск
float calk(float *Ohm, int *Grad, char count,float x,int j)
{
	int i;																				//временные переменные 
	float x1=0.0,x2=0.0,y1=0.0,y2=0.0,y=0.0;											//временные переменные 
	i=j;
	if (x<Ohm[j]) y=Grad[j];															//граничные условия (меньше min)
	else if (x>=Ohm[count-1]) y=Grad[count-1];											//больше max
	else
	{
		while (i<count-1)																//Поиск интервала 
		{
			if ((x>=Ohm[i])&&(x<(Ohm[i+1])))
			{
				x1=Ohm[i];x2=Ohm[i+1];
				y1=Grad[i];y2=Grad[i+1];
			}
			i++;
		}
		y=y1+(((y2-y1)*(x-x1))/(x2-x1));
	}
	return y;
}

//функция перерасчета сопротивления в температуру 
//float calk - значение температуры
//uint32_t adc_buf  - значение от АЦП
float calktemper(uint32_t adc_buf)
{
		float data,data_ohm;															//временные переменные 
		if (adc_buf == 0x7FFFFF)														// проверка на выход за максимальное значение диапозона
		{
			data_ohm = Pt138_Ohm[21];													// установить максимальное значение
		}
		else if (adc_buf > 0x7FFFFF)													// проверка на выход за минимальное значение диапозона
		{
			data_ohm = 0.0;																// установка значения 0
		}
		else data_ohm=((float)adc_buf*Pt138_Ohm[21])/0x7FFFFF;							// перерасчет значения от АЦП в значение сопротивления

		data=calk( Pt138_Ohm, Pt138_Grad, 22,data_ohm,0);								// функция поиска значения температуры по значению сопротивления
		return data;
}

//процедура инициализация SPI
void Init_ADS1248(void)
{
	DDRB	|= 1 << 0 |	(1<<SPI_CLK_ADS) | (1<<SPI_MOSI_ADS);							//настройка выводов контроллера для работы с SPI
	PORTB	|= 1 << 0 | (1<<SPI_CLK_ADS) | (1<<SPI_MOSI_ADS) | (1<<SPI_MISO_ADS);		//настройка выводов контроллера для работы с SPI
	
	DDRF	|=  1 << SPI_SS_ADS | 1 << SPI_RESET_ADS | 1 << SPI_START_ADS;				//настройка выводов контроллера для работы с SPI
	PORTF	|=  1 << SPI_SS_ADS | 1 << SPI_RESET_ADS | 1 << SPI_START_ADS;				//настройка выводов контроллера для работы с SPI

	SPCR = (1<<SPE)|(1<<MSTR)|(1<<CPHA) |(1<<SPR0);										//Включение SPI для работы с АЦП
}

//функция приема передачи 
//uint8_t spi - полученный байт по SPI
//uint8_t data - данные для передачи по SPI
uint8_t spi(uint8_t data)
{
	SPDR = data;												//перемещениев регистр SPI данных для отправки ведомому устройству 
	while(!(SPSR & (1<<SPIF)));									//Ожидание завершения сдвига регистра SPI
	return (SPDR);												//перемещение из регистра SPI данных от ведомого устройства
}


//процедура сброса АЦП на значения по умолчанию
void Reset_ADS1248(void)
{
	PORTF &= ~(1<<SPI_RESET_ADS);								//Управление портом для сброса АЦП к заводским настройкам
	_delay_us(1);												//Управление портом для сброса АЦП к заводским настройкам
	PORTF |= (1<<SPI_RESET_ADS);								//Управление портом для сброса АЦП к заводским настройкам
	_delay_us(20);												//Управление портом для сброса АЦП к заводским настройкам
	
	SPI_ON;														//Включение CS для микросхемы АЦП
	spi(ADS1248_COMMAND_WAKEUP);								//Команда для вывод АЦП из режима сна
	SPI_OFF;													//Выключение CS для микросхемы АЦП
}

//функция чтения данных от АЦП
uint32_t read_ADS1248(void)
{
	uint8_t DataSpi3[20];										//временая переменная для приема
	cli();														//Заперет прерываний
	_delay_us(20);	
	SPI_ON;														//Включение CS для микросхемы АЦП	
	spi(ADS1248_COMMAND_RDATA);									//Отправка команды для чтения регистров АЦП
	
	_delay_us(40);												//Задержка АЦП для подготовки АЦП результатов для чтения значения усиления  
	DataSpi3[0] = spi(0xFF);									//Чтение данных от АЦП во временную переменнутю
	DataSpi3[1] = spi(0xFF);									//Чтение данных от АЦП во временную переменнутю
	DataSpi3[2] = spi(0xFF);									//Чтение данных от АЦП во временную переменнутю
	SPI_OFF;													//Выключение CS для микросхемы АЦП	
	sei();														//Разрешение прерываний
	//_delay_us(20);
	adc_buf = 0;
	adc_buf = (uint32_t)DataSpi3[0];							//Занесение результата во временную переменную
	adc_buf = (uint32_t)DataSpi3[1] + (adc_buf*256);			//Занесение результата во временную переменную
	adc_buf = (uint32_t)DataSpi3[2] + (adc_buf*256);			//Занесение результата во временную переменную
	
	return (adc_buf);
}

//процедура записи настроек АЦП
//uint8_t type - тип настроек 
////	0 - RTD (Pt100) канал 0  
////	1 - RTD (Pt100) канал 1 
////	10 - TI (J) канал 0
////	11 - TI (J) канал 1
////	12 - TI (J) канал 2
////	13 - TI (J) канал 3
void write_ADS1248(uint8_t type)
{

	uint8_t DataSpi3TX[20];																											//временая переменная для отправки
	uint8_t DataSpi3[20];																											//временая переменная для приема
	cli();
	switch (type)																													//выбор Типа для измерения
	{
		case 0:
			SPI_ON;																													//Включение CS для микросхемы АЦП
			spi(ADS1248_COMMAND_WREG);																								//Отправка команды для записи регистров АЦП
			spi(0x0E);																												//Отправка количества учавствующих в записи регистров
			spi(ADS1248_REG_MUX0_BCS_OFF | ADS1248_REG_MUX0_MUX_SP_AIN1 | ADS1248_REG_MUX0_MUX_SN_AIN2);							//Запись значения в регистр MUX0
			spi(ADS1248_REG_VBIAS_VBIAS_OFF); 																						//Запись значения в регистр VBIAS
			spi(ADS1248_REG_MUX1_VREFCON_ALWAYS_ON | ADS1248_REG_MUX1_REFSELT_INTERNAL | ADS1248_REG_MUX1_MUXCAL_NORMAL);			//Запись значения в регистр MUX1
			spi( ADS1248_REG_SYS0_DR_160_SPS | ADS1248_REG_SYS0_PGA_4);																//Запись значения в регистр SYS0
			spi((ADCCOFF.RTD_OCR[0] >> 0) & 0xFF);																					//Запись значения в регистр OCR младшая часть
			spi((ADCCOFF.RTD_OCR[0] >> 8) & 0xFF);																					//Запись значения в регистр OCR
			spi((ADCCOFF.RTD_OCR[0] >> 16) & 0xFF);																					//Запись значения в регистр OCR старшая часть
			spi((ADCCOFF.RTD_FSR[0] >> 0) & 0xFF);																					//Запись значения в регистр FSC младшая часть
			spi((ADCCOFF.RTD_FSR[0] >> 8) & 0xFF);																					//Запись значения в регистр FSC
			spi((ADCCOFF.RTD_FSR[0] >> 16) & 0xFF);																					//Запись значения в регистр FSC старшая часть
			spi(ADS1248_REG_IDAC0_IMAG_1000mcA);																					//Запись значения в регистр IDAC0
			spi(ADS1248_REG_IDAC1_I1DIR_AIN0 | ADS1248_REG_IDAC1_I2DIR_AIN2);														//Запись значения в регистр IDAC1
			spi(ADS1248_REG_GPIOCFG_OFF);																							//Запись значения в регистр GPIOCFG
			spi(ADS1248_REG_GPIODIR_OFF);																							//Запись значения в регистр GPIODIR
			spi(ADS1248_REG_GPIODAT_OFF);																							//Запись значения в регистр GPIODAT
			SPI_OFF;																												//Выключение CS для микросхемы АЦП		
		break;
		case 1:
			SPI_ON;																													//Включение CS для микросхемы АЦП
			spi(ADS1248_COMMAND_WREG);																								//Отправка команды для записи регистров АЦП
			spi(0x0E);																												//Отправка количества учавствующих в записи регистров
			spi(ADS1248_REG_MUX0_BCS_OFF | ADS1248_REG_MUX0_MUX_SP_AIN5 | ADS1248_REG_MUX0_MUX_SN_AIN6);							//Запись значения в регистр MUX0
			spi(ADS1248_REG_VBIAS_VBIAS_OFF); 																						//Запись значения в регистр VBIAS
			spi(ADS1248_REG_MUX1_VREFCON_ALWAYS_ON | ADS1248_REG_MUX1_REFSELT_INTERNAL | ADS1248_REG_MUX1_MUXCAL_NORMAL);			//Запись значения в регистр MUX1
			spi( ADS1248_REG_SYS0_DR_160_SPS | ADS1248_REG_SYS0_PGA_4);																//Запись значения в регистр SYS0
			spi((ADCCOFF.RTD_OCR[1] >> 0) & 0xFF);																					//Запись значения в регистр OCR младшая часть
			spi((ADCCOFF.RTD_OCR[1] >> 8) & 0xFF);																					//Запись значения в регистр OCR
			spi((ADCCOFF.RTD_OCR[1] >> 16) & 0xFF);																					//Запись значения в регистр OCR старшая часть
			spi((ADCCOFF.RTD_FSR[1] >> 0) & 0xFF);																					//Запись значения в регистр FSC младшая часть
			spi((ADCCOFF.RTD_FSR[1] >> 8) & 0xFF);																					//Запись значения в регистр FSC
			spi((ADCCOFF.RTD_FSR[1] >> 16) & 0xFF);																					//Запись значения в регистр FSC старшая часть
			spi(ADS1248_REG_IDAC0_IMAG_1000mcA);																					//Запись значения в регистр IDAC0
			spi(ADS1248_REG_IDAC1_I1DIR_AIN4 | ADS1248_REG_IDAC1_I2DIR_AIN7);														//Запись значения в регистр IDAC1
			spi(ADS1248_REG_GPIOCFG_OFF);																							//Запись значения в регистр GPIOCFG
			spi(ADS1248_REG_GPIODIR_OFF);																							//Запись значения в регистр GPIODIR
			spi(ADS1248_REG_GPIODAT_OFF);																							//Запись значения в регистр GPIODAT
			SPI_OFF;																												//Выключение CS для микросхемы АЦП		
		break;
		case 10:
			SPI_ON;																													//Включение CS для микросхемы АЦП
			spi(ADS1248_COMMAND_WREG);																								//Отправка команды для записи регистров АЦП
			spi(0x0E);																												//Отправка количества учавствующих в записи регистров
			spi(ADS1248_REG_MUX0_BCS_OFF | ADS1248_REG_MUX0_MUX_SP_AIN0 | ADS1248_REG_MUX0_MUX_SN_AIN1);							//Запись значения в регистр MUX0
			spi(ADS1248_REG_VBIAS_VBIAS_OFF); 																						//Запись значения в регистр VBIAS
			spi(ADS1248_REG_MUX1_VREFCON_ALWAYS_ON | ADS1248_REG_MUX1_REFSELT_INTERNAL | ADS1248_REG_MUX1_MUXCAL_NORMAL);			//Запись значения в регистр MUX1
			spi( ADS1248_REG_SYS0_DR_1000_SPS | ADS1248_REG_SYS0_PGA_16);															//Запись значения в регистр SYS0
			spi((ADCCOFF.TI_OCR[0] >> 0) & 0xFF);																					//Запись значения в регистр OCR младшая часть
			spi((ADCCOFF.TI_OCR[0] >> 8) & 0xFF);																					//Запись значения в регистр OCR
			spi((ADCCOFF.TI_OCR[0] >> 16) & 0xFF);																					//Запись значения в регистр OCR старшая часть
			spi((ADCCOFF.TI_FSR[0] >> 0) & 0xFF);																					//Запись значения в регистр FSC младшая часть
			spi((ADCCOFF.TI_FSR[0] >> 8) & 0xFF);																					//Запись значения в регистр FSC
			spi((ADCCOFF.TI_FSR[0] >> 16) & 0xFF);																					//Запись значения в регистр FSC старшая часть
			spi(ADS1248_REG_IDAC0_IMAG_OFF);																						//Запись значения в регистр IDAC0
			spi(ADS1248_REG_IDAC1_I1DIR_DISCONNECT | ADS1248_REG_IDAC1_I2DIR_DISCONNECT);											//Запись значения в регистр IDAC1
			spi(ADS1248_REG_GPIOCFG_OFF);																							//Запись значения в регистр GPIOCFG
			spi(ADS1248_REG_GPIODIR_OFF);																							//Запись значения в регистр GPIODIR
			spi(ADS1248_REG_GPIODAT_OFF);																							//Запись значения в регистр GPIODAT
			SPI_OFF;																												//Выключение CS для микросхемы АЦП	
		break;
		case 11:
			SPI_ON;																													//Включение CS для микросхемы АЦП
			spi(ADS1248_COMMAND_WREG);																								//Отправка команды для записи регистров АЦП
			spi(0x0E);																												//Отправка количества учавствующих в записи регистров
			spi(ADS1248_REG_MUX0_BCS_OFF | ADS1248_REG_MUX0_MUX_SP_AIN2 | ADS1248_REG_MUX0_MUX_SN_AIN3);							//Запись значения в регистр MUX0
			spi(ADS1248_REG_VBIAS_VBIAS_OFF); 																						//Запись значения в регистр VBIAS
			spi(ADS1248_REG_MUX1_VREFCON_ALWAYS_ON | ADS1248_REG_MUX1_REFSELT_INTERNAL | ADS1248_REG_MUX1_MUXCAL_NORMAL);			//Запись значения в регистр MUX1
			spi( ADS1248_REG_SYS0_DR_1000_SPS | ADS1248_REG_SYS0_PGA_16);															//Запись значения в регистр SYS0
			spi((ADCCOFF.TI_OCR[1] >> 0) & 0xFF);																					//Запись значения в регистр OCR младшая часть
			spi((ADCCOFF.TI_OCR[1] >> 8) & 0xFF);																					//Запись значения в регистр OCR
			spi((ADCCOFF.TI_OCR[1] >> 16) & 0xFF);																					//Запись значения в регистр OCR старшая часть
			spi((ADCCOFF.TI_FSR[1] >> 0) & 0xFF);																					//Запись значения в регистр FSC младшая часть
			spi((ADCCOFF.TI_FSR[1] >> 8) & 0xFF);																					//Запись значения в регистр FSC
			spi((ADCCOFF.TI_FSR[1] >> 16) & 0xFF);																					//Запись значения в регистр FSC старшая часть
			spi(ADS1248_REG_IDAC0_IMAG_OFF);																						//Запись значения в регистр IDAC0
			spi(ADS1248_REG_IDAC1_I1DIR_DISCONNECT | ADS1248_REG_IDAC1_I2DIR_DISCONNECT);											//Запись значения в регистр IDAC1
			spi(ADS1248_REG_GPIOCFG_OFF);																							//Запись значения в регистр GPIOCFG
			spi(ADS1248_REG_GPIODIR_OFF);																							//Запись значения в регистр GPIODIR
			spi(ADS1248_REG_GPIODAT_OFF);																							//Запись значения в регистр GPIODAT
			SPI_OFF;																												//Выключение CS для микросхемы АЦП	
		break;
		case 12:
			SPI_ON;																													//Включение CS для микросхемы АЦП
			spi(ADS1248_COMMAND_WREG);																								//Отправка команды для записи регистров АЦП
			spi(0x0E);																												//Отправка количества учавствующих в записи регистров
			spi(ADS1248_REG_MUX0_BCS_OFF | ADS1248_REG_MUX0_MUX_SP_AIN4 | ADS1248_REG_MUX0_MUX_SN_AIN5);							//Запись значения в регистр MUX0
			spi(ADS1248_REG_VBIAS_VBIAS_OFF); 																						//Запись значения в регистр VBIAS
			spi(ADS1248_REG_MUX1_VREFCON_ALWAYS_ON | ADS1248_REG_MUX1_REFSELT_INTERNAL | ADS1248_REG_MUX1_MUXCAL_NORMAL);			//Запись значения в регистр MUX1
			spi( ADS1248_REG_SYS0_DR_1000_SPS | ADS1248_REG_SYS0_PGA_16);															//Запись значения в регистр SYS0
			spi((ADCCOFF.TI_OCR[2] >> 0) & 0xFF);																					//Запись значения в регистр OCR младшая часть
			spi((ADCCOFF.TI_OCR[2] >> 8) & 0xFF);																					//Запись значения в регистр OCR
			spi((ADCCOFF.TI_OCR[2] >> 16) & 0xFF);																					//Запись значения в регистр OCR старшая часть
			spi((ADCCOFF.TI_FSR[2] >> 0) & 0xFF);																					//Запись значения в регистр FSC младшая часть
			spi((ADCCOFF.TI_FSR[2] >> 8) & 0xFF);																					//Запись значения в регистр FSC
			spi((ADCCOFF.TI_FSR[2] >> 16) & 0xFF);																					//Запись значения в регистр FSC старшая часть
			spi(ADS1248_REG_IDAC0_IMAG_OFF);																						//Запись значения в регистр IDAC0
			spi(ADS1248_REG_IDAC1_I1DIR_DISCONNECT | ADS1248_REG_IDAC1_I2DIR_DISCONNECT);											//Запись значения в регистр IDAC1
			spi(ADS1248_REG_GPIOCFG_OFF);																							//Запись значения в регистр GPIOCFG
			spi(ADS1248_REG_GPIODIR_OFF);																							//Запись значения в регистр GPIODIR
			spi(ADS1248_REG_GPIODAT_OFF);																							//Запись значения в регистр GPIODAT
			SPI_OFF;																												//Выключение CS для микросхемы АЦП	
		break;
		case 13:
			SPI_ON;																													//Включение CS для микросхемы АЦП
			spi(ADS1248_COMMAND_WREG);																								//Отправка команды для записи регистров АЦП
			spi(0x0E);																												//Отправка количества учавствующих в записи регистров
			spi(ADS1248_REG_MUX0_BCS_OFF | ADS1248_REG_MUX0_MUX_SP_AIN6 | ADS1248_REG_MUX0_MUX_SN_AIN7);							//Запись значения в регистр MUX0
			spi(ADS1248_REG_VBIAS_VBIAS_OFF); 																						//Запись значения в регистр VBIAS
			spi(ADS1248_REG_MUX1_VREFCON_ALWAYS_ON | ADS1248_REG_MUX1_REFSELT_INTERNAL | ADS1248_REG_MUX1_MUXCAL_NORMAL);			//Запись значения в регистр MUX1
			spi( ADS1248_REG_SYS0_DR_1000_SPS | ADS1248_REG_SYS0_PGA_16);															//Запись значения в регистр SYS0
			spi((ADCCOFF.TI_OCR[3] >> 0) & 0xFF);																					//Запись значения в регистр OCR младшая часть
			spi((ADCCOFF.TI_OCR[3] >> 8) & 0xFF);																					//Запись значения в регистр OCR
			spi((ADCCOFF.TI_OCR[3] >> 16) & 0xFF);																					//Запись значения в регистр OCR старшая часть
			spi((ADCCOFF.TI_FSR[3] >> 0) & 0xFF);																					//Запись значения в регистр FSC младшая часть
			spi((ADCCOFF.TI_FSR[3] >> 8) & 0xFF);																					//Запись значения в регистр FSC
			spi((ADCCOFF.TI_FSR[3] >> 16) & 0xFF);																					//Запись значения в регистр FSC старшая часть
			spi(ADS1248_REG_IDAC0_IMAG_OFF);																						//Запись значения в регистр IDAC0
			spi(ADS1248_REG_IDAC1_I1DIR_DISCONNECT | ADS1248_REG_IDAC1_I2DIR_DISCONNECT);											//Запись значения в регистр IDAC1
			spi(ADS1248_REG_GPIOCFG_OFF);																							//Запись значения в регистр GPIOCFG
			spi(ADS1248_REG_GPIODIR_OFF);																							//Запись значения в регистр GPIODIR
			spi(ADS1248_REG_GPIODAT_OFF);																							//Запись значения в регистр GPIODAT
			SPI_OFF;																												//Выключение CS для микросхемы АЦП	
		break;
		
	}
}
//----------------------------------------------------//

//Калибровка усиления выбранного канала channal для RTD
//	uint32_t Calibrov_Max_RTD значение услиления для АЦП 
uint32_t Calibrov_Max_RTD(uint8_t channel)
{
	
	uint32_t summ = 0;										//временая переменная результата
	uint8_t DataSpi3TX[20];									//временая переменная для отправки
	uint8_t DataSpi3[20];									//временая переменная для приема
	cli();
	write_ADS1248(channel);									//запись настроек для АПЦ выбранного канала в конфигурации RTD PT100
	
	_delay_ms(1000);										//задержка для стабилизации значения перед калибровкой								
	DataSpi3TX[0]= ADS1248_COMMAND_SYSGCAL;					//занесение команды для вычисления значения усиления во временную переменную
	SPI_ON;													//Включение CS для микросхемы АЦП
	spi(DataSpi3TX[0]);										//Передача команды для вычисления значения усиления
	SPI_OFF;												//Выключение CS для микросхемы АЦП

	_delay_ms(300);											//задержка для вычисления значения усиления
	DataSpi3TX[0]= ADS1248_COMMAND_RREG | ADS1248_REG_FSC0;	//занесение команды для чтения значения усиления во временную переменную
	DataSpi3TX[1]= 0b00000010;								//занесение команды для чтения значения усиления во временную переменную

	SPI_ON; 												//Включение CS для микросхемы АЦП
	spi(DataSpi3TX[0]);										//занесение команды для чтения значения усиления							
	spi(DataSpi3TX[1]);										//занесение команды для чтения значения усиления
	_delay_us(50);											//задержка для подготовки АЦП результатов для чтения значения усиления  
	for (int i = 0; i < 3; i++) {DataSpi3[i] = spi(0xFF);}	//Чтение значения усиления
    SPI_OFF;												//Выключение CS для микросхемы АЦП

    summ = DataSpi3[2];										//Занесение результата во временную переменную
    summ = summ<<8;											//Занесение результата во временную переменную
  	summ = DataSpi3[1] + summ;								//Занесение результата во временную переменную
    summ = summ<<8;											//Занесение результата во временную переменную
    summ = DataSpi3[0] + summ;								//Занесение результата во временную переменную
	
	ADCCOFF.RTD_FSR[channel] = summ;						//Занесение результата в переменную для последующего использования в функции записи настроек в АЦП
	return summ;
}

//Калибровка смещения 0 выбранного канала channel для RTD
//	uint32_t Calibrov_Min_RTD значение смещения 0 для АЦП
uint32_t Calibrov_Min_RTD(uint8_t channel)			
{
	uint32_t summ = 0; 										//временая переменная результата
	uint8_t DataSpi3TX[20];									//временая переменная для отправки
	uint8_t DataSpi3[20];									//временая переменная для приема
	cli();
	write_ADS1248(channel); 								//запись настроек для АПЦ выбранного канала в конфигурации RTD PT100

	_delay_ms(1000);										//задержка для стабилизации значения перед калибровкой	
	DataSpi3TX[0]= ADS1248_COMMAND_SYSOCAL;					//занесение команды для вычисления значения смещения во временную переменную
	SPI_ON; 												//Включение CS для микросхемы АЦП
	spi(DataSpi3TX[0]);										//Передача команды для вычисления значения смещения
	SPI_OFF;												//Выключение CS для микросхемы АЦП
	_delay_ms(300);											//задержка для вычисления значения смещения
	DataSpi3TX[0]= ADS1248_COMMAND_RREG | ADS1248_REG_OFC0;	//занесение команды для чтения значения смещения во временную переменную
	DataSpi3TX[1]= 0b00000010;								//занесение команды для чтения значения смещения во временную переменную

	SPI_ON; 												//Включение CS для микросхемы АЦП
	spi(DataSpi3TX[0]);										//занесение команды для чтения значения смещения
	spi(DataSpi3TX[1]);										//занесение команды для чтения значения смещения
	_delay_us(50);											//задержка для подготовки АЦП результатов для чтения значения смещения  
	for (int i = 0; i < 3; i++) {DataSpi3[i] = spi(0xFF);}	//Чтение значения смещения
    SPI_OFF;												//Выключение CS для микросхемы АЦП

    summ = DataSpi3[2];										//Занесение результата во временную переменную
    summ = summ<<8;											//Занесение результата во временную переменную
  	summ = DataSpi3[1] + summ;								//Занесение результата во временную переменную
    summ = summ<<8;											//Занесение результата во временную переменную
    summ = DataSpi3[0] + summ;								//Занесение результата во временную переменную
	
	ADCCOFF.RTD_OCR[channel] = summ;						//Занесение результата в переменную для последующего использования в функции записи настроек в АЦП
	return summ;
}
//Калибровка усиления выбранного канала channel для TI
//	uint32_t Calibrov_Max_RTD значение услиления для АЦП
uint32_t Calibrov_Max_TI(uint8_t channel)
{
	
	uint32_t summ = 0;										//временая переменная результата
	uint8_t DataSpi3TX[20];									//временая переменная для отправки
	uint8_t DataSpi3[20];									//временая переменная для приема
	cli();
	write_ADS1248(10+channel); 								//запись настроек для АПЦ выбранного канала в конфигурации TI Type J
	
	_delay_ms(1000);										//задержка для стабилизации значения перед калибровкой
	DataSpi3TX[0]= ADS1248_COMMAND_SYSGCAL;					//занесение команды для вычисления значения усиления во временную переменную
	SPI_ON;													//Включение CS для микросхемы АЦП
	spi(DataSpi3TX[0]);										//Передача команды для вычисления значения усиления
	SPI_OFF;												//Выключение CS для микросхемы АЦП

	_delay_ms(300);											//задержка для вычисления значения усиления
	DataSpi3TX[0]= ADS1248_COMMAND_RREG | ADS1248_REG_FSC0;	//занесение команды для чтения значения усиления во временную переменную
	DataSpi3TX[1]= 0b00000010;								//занесение команды для чтения значения усиления во временную переменную

	SPI_ON;													//Включение CS для микросхемы АЦП
	spi(DataSpi3TX[0]);										//занесение команды для чтения значения усиления
	spi(DataSpi3TX[1]);										//занесение команды для чтения значения усиления
	_delay_us(50);											//задержка для подготовки АЦП результатов для чтения значения усиления
	for (int i = 0; i < 3; i++) {DataSpi3[i] = spi(0xFF);}	//Чтение значения усиления
	SPI_OFF;												//Выключение CS для микросхемы АЦП

	summ = DataSpi3[2];										//Занесение результата во временную переменную
	summ = summ<<8;											//Занесение результата во временную переменную
	summ = DataSpi3[1] + summ;								//Занесение результата во временную переменную
	summ = summ<<8;											//Занесение результата во временную переменную
	summ = DataSpi3[0] + summ;								//Занесение результата во временную переменную
			
	ADCCOFF.TI_FSR[channel] = summ;							//Занесение результата в переменную для последующего использования в функции записи настроек в АЦП
	
	return summ;
}

//Калибровка смещения 0 выбранного канала channel для TI
//	uint32_t Calibrov_Min_RTD значение смещения 0 для АЦП
uint32_t Calibrov_Min_TI(uint8_t channel)
{
	
	uint32_t summ = 0; 										//временая переменная результата
	uint8_t DataSpi3TX[20];									//временая переменная для отправки
	uint8_t DataSpi3[20];									//временая переменная для приема
	cli();
	write_ADS1248(10+channel);	 							//запись настроек для АПЦ выбранного канала в конфигурации TI Type J
	
	_delay_ms(1000);										//задержка для стабилизации значения перед калибровкой
	DataSpi3TX[0]= ADS1248_COMMAND_SYSOCAL;					//занесение команды для вычисления значения смещения во временную переменную
	SPI_ON; 												//Включение CS для микросхемы АЦП
	spi(DataSpi3TX[0]);										//Передача команды для вычисления значения смещения
	SPI_OFF;												//Выключение CS для микросхемы АЦП
	_delay_ms(300);											//задержка для вычисления значения смещения
	DataSpi3TX[0]= ADS1248_COMMAND_RREG | ADS1248_REG_OFC0;	//занесение команды для чтения значения смещения во временную переменную
	DataSpi3TX[1]= 0b00000010;								//занесение команды для чтения значения смещения во временную переменную

	SPI_ON; 												//Включение CS для микросхемы АЦП
	spi(DataSpi3TX[0]);										//занесение команды для чтения значения смещения
	spi(DataSpi3TX[1]);										//занесение команды для чтения значения смещения
	_delay_us(50);											//задержка для подготовки АЦП результатов для чтения значения смещения
	for (int i = 0; i < 3; i++) {DataSpi3[i] = spi(0xFF);}	//Чтение значения смещения
	SPI_OFF;												//Выключение CS для микросхемы АЦП

	summ = DataSpi3[2];										//Занесение результата во временную переменную
	summ = summ<<8;											//Занесение результата во временную переменную
	summ = DataSpi3[1] + summ;								//Занесение результата во временную переменную
	summ = summ<<8;											//Занесение результата во временную переменную
	summ = DataSpi3[0] + summ;								//Занесение результата во временную переменную

	ADCCOFF.TI_OCR[channel] = summ;							//Занесение результата в переменную для последующего использования в функции записи настроек в АЦП
	
	return summ;
}

//функция перевода полученного значения напряжения от термопары в температуру в соответствии с таблице ГОСТ термопары типа J
// float voltage - напряжение
// float TI_to_temperature() - температура
float TI_to_temperature(float voltage)
{
	float temperature = 0.0;																//	Временная переменная
	float coefficient[10];																	//	Временная переменная
	if(voltage < -8.095)	{temperature = -INFINITY;}										//	Установка значение -INF в случае если напряжение меньше минимального значения  
	else if(voltage < 0.0)																	//	Расчет значения в диапазоне от -8.095 до 0.000 с помощью аппроксимированной функции согласно ГОСТ Р 8.585-2001 для типа J
	{
		coefficient[0] = 0;
		coefficient[1] = 19.528268;
		coefficient[2] = -1.2286185;
		coefficient[3] = -1.0752178;
		coefficient[4] = -0.59086933;
		coefficient[5] = -0.17256713;
		coefficient[6] = -2.8131513 * powf(10, -2);
		coefficient[7] = -2.3963370 * powf(10, -3);
		coefficient[8] = -8.3823321 * powf(10, -5);

		for (int j=0;j<9;j++) {temperature += coefficient[j] * powf(voltage, j);}
	}
	else if(voltage < 42.919)																//	Расчет значения в диапазоне от 0.000 до 42.919 с помощью аппроксимированной функции согласно ГОСТ Р 8.585-2001 для типа J
	{
		coefficient[0] = 0;
		coefficient[1] = 19.78425;
		coefficient[2] = -0.2001204;
		coefficient[3] = 1.036969 * powf(10, -2);
		coefficient[4] = -2.549687* powf(10, -4);
		coefficient[5] = 3.585153 * powf(10, -6);
		coefficient[6] = -5.344285 * powf(10, -8);
		coefficient[7] = 5.099890 * powf(10, -10);

		for (int j=0;j<8;j++) {temperature += coefficient[j] * powf(voltage, j);}
	}
	else if(voltage < 69.553)																//	Расчет значения в диапазоне от 42.919 до 69.553 с помощью аппроксимированной функции согласно ГОСТ Р 8.585-2001 для типа J
	{
		coefficient[0] = -3113.58187;
		coefficient[1] = 300.543684;
		coefficient[2] = -9.94773230;
		coefficient[3] = 0.170276630;
		coefficient[4] = -1.43033468 * powf(10, -3);
		coefficient[5] = 4.73886084 * powf(10, -6);
		for (int j=0;j<6;j++) {temperature += coefficient[j] * powf(voltage, j);}
	}
	else  { temperature = INFINITY;}														//	Установка значение +INF в случае если напряжение больше максимального значения 
	return temperature;
}