/*
 * ADS1248.h
 *
 *  Created on: May 6, 2021
 *      Author: dubovik
 */

#ifndef INC_ADS1248_H_
#define INC_ADS1248_H_

#define F_CPU 14745600UL 

#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>

#define SPI_SS_ADS 1
#define SPI_RESET_ADS 2
#define SPI_START_ADS 0
#define SPI_MOSI_ADS 2
#define SPI_MISO_ADS 3
#define SPI_CLK_ADS 1

#define SPI_PORTX PORTF
#define SPI_DDRX DDRF
//Тип настроек
#define TYPE_SETTINGS_FOR_RTD_CH_0 				0x00
#define TYPE_SETTINGS_FOR_RTD_CH_1 				0x01
#define TYPE_SETTINGS_FOR_TI_CH_0 				0x10
#define TYPE_SETTINGS_FOR_TI_CH_1 				0x11
#define TYPE_SETTINGS_FOR_TI_CH_2 				0x12
#define TYPE_SETTINGS_FOR_TI_CH_3 				0x13

//Адреса регистров
#define ADS1248_REG_MUX0 				0x00
#define ADS1248_REG_VBIAS 				0x01
#define ADS1248_REG_MUX1 				0x02
#define ADS1248_REG_SYS0 				0x03
#define ADS1248_REG_OFC0 				0x04
#define ADS1248_REG_OFC1 				0x05
#define ADS1248_REG_OFC2 				0x06
#define ADS1248_REG_FSC0	 			0x07
#define ADS1248_REG_FSC1 				0x08
#define ADS1248_REG_FSC2 				0x09
#define ADS1248_REG_IDAC0	 			0x0A
#define ADS1248_REG_IDAC1 				0x0B
#define ADS1248_REG_GPIOCFG 			0x0C
#define ADS1248_REG_GPIODIR 			0x0D
#define ADS1248_REG_GPIODAT 			0x0E
//Коды команд
#define ADS1248_COMMAND_WAKEUP 			0x00
#define ADS1248_COMMAND_WAKEUP1 		0x01
#define ADS1248_COMMAND_SLEEP			0x02
#define ADS1248_COMMAND_SLEEP1 			0x03
#define ADS1248_COMMAND_SYNC			0x04
#define ADS1248_COMMAND_SYNC1 			0x05
#define ADS1248_COMMAND_RESET			0x06
#define ADS1248_COMMAND_RESET1 			0x07
#define ADS1248_COMMAND_RDATA			0x12
#define ADS1248_COMMAND_RDATA1 			0x13
#define ADS1248_COMMAND_RDATAC			0x14
#define ADS1248_COMMAND_RDATAC1 		0x15
#define ADS1248_COMMAND_SDATAC			0x16
#define ADS1248_COMMAND_SDATAC1 		0x17
#define ADS1248_COMMAND_RREG			0x20
#define ADS1248_COMMAND_WREG 			0x40
#define ADS1248_COMMAND_SYSOCAL			0x60
#define ADS1248_COMMAND_SYSGCAL 		0x61
#define ADS1248_COMMAND_SELFOCAL		0x62
#define ADS1248_COMMAND_NOP 			0xFF
//Значение для регистра MUX0
#define ADS1248_REG_MUX0_MUX_SN_AIN0			0x00
#define ADS1248_REG_MUX0_MUX_SN_AIN1			0x01
#define ADS1248_REG_MUX0_MUX_SN_AIN2			0x02
#define ADS1248_REG_MUX0_MUX_SN_AIN3			0x03
#define ADS1248_REG_MUX0_MUX_SN_AIN4			0x04
#define ADS1248_REG_MUX0_MUX_SN_AIN5			0x05
#define ADS1248_REG_MUX0_MUX_SN_AIN6			0x06
#define ADS1248_REG_MUX0_MUX_SN_AIN7			0x07
#define ADS1248_REG_MUX0_MUX_SP_AIN0			0x00
#define ADS1248_REG_MUX0_MUX_SP_AIN1			0x08
#define ADS1248_REG_MUX0_MUX_SP_AIN2			0x10
#define ADS1248_REG_MUX0_MUX_SP_AIN3			0x18
#define ADS1248_REG_MUX0_MUX_SP_AIN4			0x20
#define ADS1248_REG_MUX0_MUX_SP_AIN5			0x28
#define ADS1248_REG_MUX0_MUX_SP_AIN6			0x30
#define ADS1248_REG_MUX0_MUX_SP_AIN7			0x38
#define ADS1248_REG_MUX0_BCS_OFF				0x00
#define ADS1248_REG_MUX0_BCS_ON_0_5mcA			0x40
#define ADS1248_REG_MUX0_BCS_ON_2mcA			0x80
#define ADS1248_REG_MUX0_BCS_ON_10mcA			0xС0
//Значение для регистра VBIAS
#define ADS1248_REG_VBIAS_VBIAS_OFF				0x00
#define ADS1248_REG_VBIAS_VBIAS_AIN0			0x01
#define ADS1248_REG_VBIAS_VBIAS_AIN1			0x02
#define ADS1248_REG_VBIAS_VBIAS_AIN2			0x04
#define ADS1248_REG_VBIAS_VBIAS_AIN3			0x08
#define ADS1248_REG_VBIAS_VBIAS_AIN4			0x10
#define ADS1248_REG_VBIAS_VBIAS_AIN5			0x20
#define ADS1248_REG_VBIAS_VBIAS_AIN6			0x40
#define ADS1248_REG_VBIAS_VBIAS_AIN7			0x80
//Значение для регистра MUX1
#define ADS1248_REG_MUX1_MUXCAL_NORMAL					0x00
#define ADS1248_REG_MUX1_MUXCAL_OFFSET_CALIBRATION		0x01
#define ADS1248_REG_MUX1_MUXCAL_GAIN_CALIBRATION		0x02
#define ADS1248_REG_MUX1_MUXCAL_TEMPERATURE_MEAS		0x03
#define ADS1248_REG_MUX1_MUXCAL_REF1_MONITOR			0x04
#define ADS1248_REG_MUX1_MUXCAL_REF0_MONITOR			0x05
#define ADS1248_REG_MUX1_MUXCAL_ANALOG_SUPPLY			0x06
#define ADS1248_REG_MUX1_MUXCAL_DIGITAL_SUPPLY			0x07
#define ADS1248_REG_MUX1_REFSELT_REFN0					0x00
#define ADS1248_REG_MUX1_REFSELT_REFN1					0x08
#define ADS1248_REG_MUX1_REFSELT_INTERNAL				0x10
#define ADS1248_REG_MUX1_REFSELT_INTERNAL_REFN0			0x18
#define ADS1248_REG_MUX1_VREFCON_ALWAYS_OFF				0x00
#define ADS1248_REG_MUX1_VREFCON_ALWAYS_ON				0x20
#define ADS1248_REG_MUX1_VREFCON_CONVERSION				0x40
#define ADS1248_REG_MUX1_CLKSTAT						0x80
//Значение для регистра SYS0
#define ADS1248_REG_SYS0_DR_5_SPS			0x00
#define ADS1248_REG_SYS0_DR_10_SPS			0x01
#define ADS1248_REG_SYS0_DR_20_SPS			0x02
#define ADS1248_REG_SYS0_DR_40_SPS			0x03
#define ADS1248_REG_SYS0_DR_80_SPS			0x04
#define ADS1248_REG_SYS0_DR_160_SPS			0x05
#define ADS1248_REG_SYS0_DR_320_SPS			0x06
#define ADS1248_REG_SYS0_DR_640_SPS			0x07
#define ADS1248_REG_SYS0_DR_1000_SPS		0x08
#define ADS1248_REG_SYS0_DR_2000_SPS		0x09
#define ADS1248_REG_SYS0_PGA_1				0x00
#define ADS1248_REG_SYS0_PGA_2				0x10
#define ADS1248_REG_SYS0_PGA_4				0x20
#define ADS1248_REG_SYS0_PGA_8				0x30
#define ADS1248_REG_SYS0_PGA_16				0x40
#define ADS1248_REG_SYS0_PGA_32				0x50
#define ADS1248_REG_SYS0_PGA_64				0x60
#define ADS1248_REG_SYS0_PGA_128			0x70
//Значение для регистра IDAC0
#define ADS1248_REG_IDAC0_IMAG_OFF			0x00
#define ADS1248_REG_IDAC0_IMAG_50mcA		0x01
#define ADS1248_REG_IDAC0_IMAG_100mcA		0x02
#define ADS1248_REG_IDAC0_IMAG_250mcA		0x03
#define ADS1248_REG_IDAC0_IMAG_500mcA		0x04
#define ADS1248_REG_IDAC0_IMAG_750mcA		0x05
#define ADS1248_REG_IDAC0_IMAG_1000mcA		0x06
#define ADS1248_REG_IDAC0_IMAG_1500mcA		0x07
#define ADS1248_REG_IDAC0_DRDYMODE			0x08
//Значение для регистра IDAC1
#define ADS1248_REG_IDAC1_I2DIR_AIN0			0x00
#define ADS1248_REG_IDAC1_I2DIR_AIN1			0x01
#define ADS1248_REG_IDAC1_I2DIR_AIN2			0x02
#define ADS1248_REG_IDAC1_I2DIR_AIN3			0x03
#define ADS1248_REG_IDAC1_I2DIR_AIN4			0x04
#define ADS1248_REG_IDAC1_I2DIR_AIN5			0x05
#define ADS1248_REG_IDAC1_I2DIR_AIN6			0x06
#define ADS1248_REG_IDAC1_I2DIR_AIN7			0x07
#define ADS1248_REG_IDAC1_I2DIR_IEXC1			0x08
#define ADS1248_REG_IDAC1_I2DIR_IEXC2			0x09
#define ADS1248_REG_IDAC1_I2DIR_DISCONNECT		0x0C
#define ADS1248_REG_IDAC1_I1DIR_AIN0			0x00
#define ADS1248_REG_IDAC1_I1DIR_AIN1			0x10
#define ADS1248_REG_IDAC1_I1DIR_AIN2			0x20
#define ADS1248_REG_IDAC1_I1DIR_AIN3			0x30
#define ADS1248_REG_IDAC1_I1DIR_AIN4			0x40
#define ADS1248_REG_IDAC1_I1DIR_AIN5			0x50
#define ADS1248_REG_IDAC1_I1DIR_AIN6			0x60
#define ADS1248_REG_IDAC1_I1DIR_AIN7			0x70
#define ADS1248_REG_IDAC1_I1DIR_IEXC1			0x80
#define ADS1248_REG_IDAC1_I1DIR_IEXC2			0x90
#define ADS1248_REG_IDAC1_I1DIR_DISCONNECT		0xC0
//Значение для регистра GPIOCFG
#define ADS1248_REG_GPIOCFG_OFF				0x00
#define ADS1248_REG_GPIOCFG_IOCFG0			0x01
#define ADS1248_REG_GPIOCFG_IOCFG1			0x02
#define ADS1248_REG_GPIOCFG_IOCFG2			0x04
#define ADS1248_REG_GPIOCFG_IOCFG3			0x08
#define ADS1248_REG_GPIOCFG_IOCFG4			0x10
#define ADS1248_REG_GPIOCFG_IOCFG5			0x20
#define ADS1248_REG_GPIOCFG_IOCFG6			0x40
#define ADS1248_REG_GPIOCFG_IOCFG7			0x80
//Значение для регистра GPIODIR
#define ADS1248_REG_GPIODIR_OFF				0x00
#define ADS1248_REG_GPIODIR_IODIR0			0x01
#define ADS1248_REG_GPIODIR_IODIR1			0x02
#define ADS1248_REG_GPIODIR_IODIR2			0x04
#define ADS1248_REG_GPIODIR_IODIR3			0x08
#define ADS1248_REG_GPIODIR_IODIR4			0x10
#define ADS1248_REG_GPIODIR_IODIR5			0x20
#define ADS1248_REG_GPIODIR_IODIR6			0x40
#define ADS1248_REG_GPIODIR_IODIR7			0x80
//Значение для регистра GPIODAT
#define ADS1248_REG_GPIODAT_OFF				0x00
#define ADS1248_REG_GPIODAT_IODAT0			0x01
#define ADS1248_REG_GPIODAT_IODAT1			0x02
#define ADS1248_REG_GPIODAT_IODAT2			0x04
#define ADS1248_REG_GPIODAT_IODAT3			0x08
#define ADS1248_REG_GPIODAT_IODAT4			0x10
#define ADS1248_REG_GPIODAT_IODAT5			0x20
#define ADS1248_REG_GPIODAT_IODAT6			0x40
#define ADS1248_REG_GPIODAT_IODAT7			0x80

uint32_t read_ADS1248(void);
void write_ADS1248(uint8_t type);
uint32_t Calibrov_Max_RTD(uint8_t channel);
uint32_t Calibrov_Min_RTD(uint8_t channel);
uint32_t Calibrov_Max_TI(uint8_t channel);
uint32_t Calibrov_Min_TI(uint8_t channel);
void Init_ADS1248(void);
void Reset_ADS1248(void);
float calktemper(uint32_t adc_buf);
float TI_to_temperature(float voltage);

#define  BUFFERSIZEFLASH32COEF 100

typedef union{
	struct{
		uint32_t RTD_OCR[5];
		uint32_t RTD_FSR[5];
		uint32_t TI_OCR[20];
		uint32_t TI_FSR[20];
		uint32_t TI_OCR_CJ;
		uint32_t TI_FSR_CJ;
	};

	uint32_t data[BUFFERSIZEFLASH32COEF];
}coef_t;


#endif /* INC_ADS1248_H_ */
